@@ -30,6 +30,7 @@
 #include "nvc_internal.h"
 
 #include "error.h"
+#include "options.h"
 #include "utils.h"
 #include "xfuncs.h"
 
@@ -471,7 +472,19 @@ nvc_ldcache_update(struct nvc_context *ctx, const struct nvc_container *cnt)
         if (validate_args(ctx, cnt != NULL) < 0)
                 return (-1);
 
-        argv = (char * []){cnt->cfg.ldconfig, "-f", "/etc/ld.so.conf", "-C", "/etc/ld.so.cache", cnt->cfg.libs_dir, cnt->cfg.libs32_dir, NULL};
+        if (cnt->flags & OPT_CUDA_COMPAT_MODE_LDCONFIG && cnt->cuda_compat_dir != NULL) {
+                /*
+                 * We include the cuda_compat_dir directory on the ldconfig
+                 * command line. This ensures that the CUDA Forward compat
+                 * libraries take precendence over the user-mode driver
+                 * libraries in the standard library paths (libs_dir and
+                 * libs32_dir).
+                 * */
+                argv = (char * []){cnt->cfg.ldconfig, "-f", "/etc/ld.so.conf", "-C", "/etc/ld.so.cache", cnt->cuda_compat_dir, cnt->cfg.libs_dir, cnt->cfg.libs32_dir, NULL};
+        } else {
+                argv = (char * []){cnt->cfg.ldconfig, "-f", "/etc/ld.so.conf", "-C", "/etc/ld.so.cache", cnt->cfg.libs_dir, cnt->cfg.libs32_dir, NULL};
+        }
+
         if (*argv[0] == '@') {
                 /*
                  * We treat this path specially to be relative to the host filesystem.
